// ======================================================================
//
// gl_d3d_5.h
// jeff grills
//
// copyright 1998 origin systems inc
//
// ======================================================================

#ifndef GL_D3D_5_H
#define GL_D3D_5_H

// ======================================================================
// includes

#include "camera.h"
#include "Gl_dd.h"
#include "Direct3d_5.h"
#include "Geometry.h"

class  MaterialData_d3d_5;
class  Material;
class  MemoryBlockManager;

// ======================================================================
// defines

class GL_d3d_5 : public GL_dd
{
private:

	// disable copy constructor and assignment operator
	GL_d3d_5(const GL_d3d_5 &);
	GL_d3d_5 &operator =(const GL_d3d_5 &);

private:

	static int                     locked;
	static int                     tripleBuffer;
	static u_int                   currentMaterialToken;
	static const Direct3d::Driver *driver;

	static int                     matR;
	static int                     matG;
	static int                     matB;

	static LPDIRECT3DDEVICE2       device;
	static LPDIRECT3DVIEWPORT2     viewport;
	static LPDIRECT3DMATERIAL2     background;
	static D3DTLVERTEX             vertexArray[Geometry::TVERTEX_BUFFER];

	static bool                    bARGB_0555;
	static bool                    bARGB_1555;
	static bool                    bARGB_4444;

	static bool                    materialTranslucent;
	static PrimitiveProcessing     primitiveProcessing;

private:

	static HRESULT WINAPI enumTextureFormatsCallback(LPDDSURFACEDESC pddsd, LPVOID lpContext);

	static const Direct3d::Driver *pickDriver(int minor);
	static void  getCaps(LPDIRECT3DDEVICE2 device);
	static void  createZBuffer(LPDIRECTDRAWSURFACE surface, int width, int height);

	static void  setSolid(void);

	static int  getD3DBlend(AlphaBlendFunction function);

	static void  setViewport(int x0, int y0, int x1, int y1);

	// submit transformed/clipped primitives to the rasterizer
	static void passPointList(const TVertex *vertexList, int numberOfVertices);
	static void passLineList(const TVertex *vertexList, int numberOfVertices);
	static void passLineStrip(const TVertex *vertexList, int numberOfVertices);
	static void passTriangleList(const TVertex *vertexList, int numberOfVertices);
	static void passTriangleStrip(const TVertex *vertexList, int numberOfVertices);
	static void passTriangleFan(const TVertex *vertexList, int numberOfVertices);

	static void testPointList(const TVertex *vertexList, int numberOfVertices);
	static void testLineList(const TVertex *vertexList, int numberOfVertices);
	static void testLineStrip(const TVertex *vertexList, int numberOfVertices);
	static void testTriangleList(const TVertex *vertexList, int numberOfVertices);
	static void testTriangleStrip(const TVertex *vertexList, int numberOfVertices);
	static void testTriangleFan(const TVertex *vertexList, int numberOfVertices);

	static void queuePointList(const TVertex *vertexList, int numberOfVertices);
	static void queueLineList(const TVertex *vertexList, int numberOfVertices);
	static void queueLineStrip(const TVertex *vertexList, int numberOfVertices);
	static void queueTriangleList(const TVertex *vertexList, int numberOfVertices);
	static void queueTriangleStrip(const TVertex *vertexList, int numberOfVertices);
	static void queueTriangleFan(const TVertex *vertexList, int numberOfVertices);

public:

	static LPDIRECT3DDEVICE2 getDevice(void);

	static  bool isHardware(void);
	static  bool supports0555(void);
	static  bool supports1555(void);
	static  bool supports4444(void);

public:

	GL_d3d_5(void);
	virtual ~GL_d3d_5(void);

	// version information
	virtual void  getInformation(Information *information);

	// see if the is usable
	virtual bool  query(int minor);

	// initialize
	virtual bool init(const GL_input *input, GL_output *output);

	// application activation
	virtual void  setActive(bool active);

	// access functions
	virtual bool lock(void);
	virtual void unlock(bool display);

	// set processing of primitives
	virtual void setAlphaBlendEnabled(bool enabled);
	virtual void setAlphaBlendFunctions(AlphaBlendFunction source, AlphaBlendFunction destination);
	virtual void setPrimitiveProcessing(PrimitiveProcessing primitiveProcessing);

	// palette functions
	virtual PaletteData *paletteCreateData(const Palette *pal);
	virtual void         paletteDestroyData(PaletteData *palData);

	// material functions
	virtual MaterialData *materialCreateData(const StaticMaterial *material);
	virtual void          materialDestroyData(MaterialData *mat);

	// camera render stuff
	virtual void beginRender(const Camera &camera);
	virtual void endRender(void);

	// 3d render setup
	virtual void setLightingEnabled(bool lightingEnabled);
	virtual void setLightList(Light **lightList, int lightCount);
	virtual bool setVertexList(const Vector *vertex, int vertexCount, const Vector *normal, int normalCount);
	virtual void setObjectToWorld(const Xform &objectToWorld);

	// 3d polygon functions
	virtual void setMaterial(const StaticMaterial *material);

	// object-space primitives
	virtual void drawPointList(const UVertex *vertexList, int numberOfVertices);
	virtual void drawLineList(const UVertex *vertexList, int numberOfVertices);
	virtual void drawLineStrip(const UVertex *vertexList, int numberOfVertices);
	virtual void drawTriangleList(const UVertex *vertexList, int numberOfVertices);
	virtual void drawTriangleStrip(const UVertex *vertexList, int numberOfVertices);
	virtual void drawTriangleFan(const UVertex *vertexList, int numberOfVertices);

	// screen-space primitives
	virtual void drawPointList(const TVertex *vertexList, int numberOfVertices);
	virtual void drawLineList(const TVertex *vertexList, int numberOfVertices);
	virtual void drawLineStrip(const TVertex *vertexList, int numberOfVertices);
	virtual void drawTriangleList(const TVertex *vertexList, int numberOfVertices);
	virtual void drawTriangleStrip(const TVertex *vertexList, int numberOfVertices);
	virtual void drawTriangleFan(const TVertex *vertexList, int numberOfVertices);

	virtual void movie(const uv_instance *uv);
	virtual bool specialEffect(int id, void *data);
};

// ======================================================================

inline bool GL_d3d_5::isHardware(void)
{
	return driver->isHardware;
}

// ----------------------------------------------------------------------

inline bool GL_d3d_5::supports0555(void)
{
	return bARGB_0555;
}

// ----------------------------------------------------------------------

inline bool GL_d3d_5::supports1555(void)
{
	return bARGB_1555;
}

// ----------------------------------------------------------------------

inline bool GL_d3d_5::supports4444(void)
{
	return bARGB_4444;
}

// ----------------------------------------------------------------------

inline LPDIRECT3DDEVICE2 GL_d3d_5::getDevice(void)
{
	return device;
}

// ======================================================================

#endif

